@@ -2,21 +2,22 @@
  * Copyright (C) 1996,1997,1998,1999,2000,2001, 2003, 2004, 2006 Free Software Foundation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
@@ -47,6 +48,7 @@
 #include "libguile/lang.h"
 #include "libguile/backtrace.h"
 #include "libguile/filesys.h"
+#include "libguile/private-options.h"
 
 /* {Error reporting and backtraces}
  *
@@ -466,8 +468,21 @@ static void
 display_backtrace_get_file_line (SCM frame, SCM *file, SCM *line)
 {
   SCM source = SCM_FRAME_SOURCE (frame);
-  *file = SCM_MEMOIZEDP (source) ? scm_source_property (source, scm_sym_filename) : SCM_BOOL_F;
-  *line = (SCM_MEMOIZEDP (source)) ? scm_source_property (source, scm_sym_line) : SCM_BOOL_F;
+  *file = *line = SCM_BOOL_F;
+  if (SCM_MEMOIZEDP (source))
+    {
+      *file = scm_source_property (source, scm_sym_filename);
+      *line = scm_source_property (source, scm_sym_line);
+    }
+  else if (scm_is_pair (source)
+           && scm_is_pair (scm_cdr (source))
+           && scm_is_pair (scm_cddr (source))
+           && !scm_is_pair (scm_cdddr (source)))
+    {
+      /* (addr . (filename . (line . column))), from vm compilation */
+      *file = scm_cadr (source);
+      *line = scm_caddr (source);
+    }
 }
 
 static void
@@ -730,14 +745,14 @@ display_backtrace_body (struct display_backtrace_args *a)
 
 SCM_DEFINE (scm_display_backtrace_with_highlights, "display-backtrace", 2, 3, 0, 
 	    (SCM stack, SCM port, SCM first, SCM depth, SCM highlights),
-	    "Display a backtrace to the output port @var{port}. @var{stack}\n"
+	    "Display a backtrace to the output port @var{port}.  @var{stack}\n"
 	    "is the stack to take the backtrace from, @var{first} specifies\n"
-	    "where in the stack to start and @var{depth} how much frames\n"
-	    "to display. Both @var{first} and @var{depth} can be @code{#f},\n"
+	    "where in the stack to start and @var{depth} how many frames\n"
+	    "to display.  @var{first} and @var{depth} can be @code{#f},\n"
 	    "which means that default values will be used.\n"
-	    "When @var{highlights} is given,\n"
-	    "it should be a list and all members of it are highligthed in\n"
-	    "the backtrace.")
+	    "If @var{highlights} is given it should be a list; the elements\n"
+	    "of this list will be highlighted wherever they appear in the\n"
+	    "backtrace.")
 #define FUNC_NAME s_scm_display_backtrace_with_highlights
 {
   struct display_backtrace_args a;
@@ -771,9 +786,9 @@ SCM_VARIABLE (scm_has_shown_backtrace_hint_p_var, "has-shown-backtrace-hint?");
 SCM_DEFINE (scm_backtrace_with_highlights, "backtrace", 0, 1, 0, 
 	    (SCM highlights),
 	    "Display a backtrace of the stack saved by the last error\n"
-	    "to the current output port.  When @var{highlights} is given,\n"
-	    "it should be a list and all members of it are highligthed in\n"
-	    "the backtrace.")
+	    "to the current output port.  If @var{highlights} is given\n"
+	    "it should be a list; the elements of this list will be\n"
+	    "highlighted wherever they appear in the backtrace.")
 #define FUNC_NAME s_scm_backtrace_with_highlights
 {
   SCM port = scm_current_output_port ();
