@@ -1,25 +1,30 @@
 /* dynl.c - dynamic linking
  *
  * Copyright (C) 1990, 91, 92, 93, 94, 95, 96, 97, 98, 99, 2000, 2001, 2002,
- * 2003 Free Software Foundation, Inc.
+ * 2003, 2008 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 /* "dynl.c" dynamically link&load object files.
    Author: Aubrey Jaffer
    Modified for libguile by Marius Vollmer */
@@ -64,6 +69,7 @@ maybe_drag_in_eprintf ()
   SCM_CRITICAL_SECTION_END were used: they are mentioned here in case
   somebody is grepping for thread problems ;)
 */
+/* njrev: not threadsafe, protection needed as described above */
 
 static void *
 sysdep_dynl_link (const char *fname, const char *subr)
@@ -152,11 +158,11 @@ SCM_DEFINE (scm_dynamic_link, "dynamic-link", 1, 0, 0,
   void *handle;
   char *file;
 
-  scm_frame_begin (0);
+  scm_dynwind_begin (0);
   file = scm_to_locale_string (filename);
-  scm_frame_free (file);
+  scm_dynwind_free (file);
   handle = sysdep_dynl_link (file, FUNC_NAME);
-  scm_frame_end ();
+  scm_dynwind_end ();
   SCM_RETURN_NEWSMOB2 (scm_tc16_dynamic_obj, SCM_UNPACK (filename), handle);
 }
 #undef FUNC_NAME
@@ -221,12 +227,12 @@ SCM_DEFINE (scm_dynamic_func, "dynamic-func", 2, 0, 0,
   } else {
     char *chars;
 
-    scm_frame_begin (0);
+    scm_dynwind_begin (0);
     chars = scm_to_locale_string (name);
-    scm_frame_free (chars);
+    scm_dynwind_free (chars);
     func = (void (*) ()) sysdep_dynl_func (chars, DYNL_HANDLE (dobj), 
 					   FUNC_NAME);
-    scm_frame_end ();
+    scm_dynwind_end ();
     return scm_from_ulong ((unsigned long) func);
   }
 }
@@ -289,7 +295,7 @@ SCM_DEFINE (scm_dynamic_args_call, "dynamic-args-call", 3, 0, 0,
   int result, argc;
   char **argv;
 
-  scm_frame_begin (0);
+  scm_dynwind_begin (0);
 
   if (scm_is_string (func))
     func = scm_dynamic_func (func, dobj);
@@ -297,13 +303,13 @@ SCM_DEFINE (scm_dynamic_args_call, "dynamic-args-call", 3, 0, 0,
   fptr = (int (*) (int, char **)) scm_to_ulong (func);
 
   argv = scm_i_allocate_string_pointers (args);
-  scm_frame_unwind_handler (free_string_pointers, argv,
-			    SCM_F_WIND_EXPLICITLY);
+  scm_dynwind_unwind_handler (free_string_pointers, argv,
+			      SCM_F_WIND_EXPLICITLY);
   for (argc = 0; argv[argc]; argc++)
     ;
   result = (*fptr) (argc, argv);
 
-  scm_frame_end ();
+  scm_dynwind_end ();
   return scm_from_int (result);
 }
 #undef FUNC_NAME
