@@ -1,25 +1,24 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2006 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2006, 2008 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
-#define _GNU_SOURCE
-
 /* #define DEBUGINFO */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
@@ -145,7 +144,7 @@ scm_assert_cell_valid (SCM cell)
       */
       if (scm_expensive_debug_cell_accesses_p)
 	scm_i_expensive_validation_check (cell);
-      
+#if (SCM_DEBUG_MARKING_API == 0)
       if (!SCM_GC_MARK_P (cell))
 	{
 	  fprintf (stderr,
@@ -155,7 +154,8 @@ scm_assert_cell_valid (SCM cell)
                    (unsigned long) SCM_UNPACK (cell));
 	  abort ();
 	}
-
+#endif /* SCM_DEBUG_MARKING_API */
+      
       scm_i_cell_validation_already_running = 0;  /* re-enable */
     }
 }
@@ -208,23 +208,29 @@ unsigned long scm_mtrigger;
 /* GC Statistics Keeping
  */
 unsigned long scm_cells_allocated = 0;
-unsigned long scm_last_cells_allocated;
+unsigned long scm_last_cells_allocated = 0;
 unsigned long scm_mallocated = 0;
-unsigned long scm_gc_cells_collected;
-unsigned long scm_gc_cells_collected_1 = 0; /* previous GC yield */
-unsigned long scm_gc_malloc_collected;
-unsigned long scm_gc_ports_collected;
-unsigned long scm_gc_time_taken = 0;
-static unsigned long t_before_gc;
-unsigned long scm_gc_mark_time_taken = 0;
-unsigned long scm_gc_times = 0;
-unsigned long scm_gc_cells_swept = 0;
-double scm_gc_cells_marked_acc = 0.;
-double scm_gc_cells_swept_acc = 0.;
-double scm_gc_cells_allocated_acc = 0.;
-int scm_gc_cell_yield_percentage =0;
+long int scm_i_find_heap_calls = 0;
+/* Global GC sweep statistics since the last full GC.  */
+scm_t_sweep_statistics scm_i_gc_sweep_stats = { 0, 0 };
+
+/* Total count of cells marked/swept.  */
+static double scm_gc_cells_marked_acc = 0.;
+static double scm_gc_cells_marked_conservatively_acc = 0.;
+static double scm_gc_cells_swept_acc = 0.;
+static double scm_gc_cells_allocated_acc = 0.;
+
+static unsigned long scm_gc_time_taken = 0;
+static unsigned long scm_gc_mark_time_taken = 0;
+
+static unsigned long scm_gc_times = 0;
+
+static int scm_gc_cell_yield_percentage = 0;
+static unsigned long protected_obj_count = 0;
+
+/* The following are accessed from `gc-malloc.c' and `gc-card.c'.  */
 int scm_gc_malloc_yield_percentage = 0;
-unsigned long protected_obj_count = 0;
+unsigned long scm_gc_malloc_collected = 0;
 
 
 SCM_SYMBOL (sym_cells_allocated, "cells-allocated");
@@ -236,6 +242,7 @@ SCM_SYMBOL (sym_gc_time_taken, "gc-time-taken");
 SCM_SYMBOL (sym_gc_mark_time_taken, "gc-mark-time-taken");
 SCM_SYMBOL (sym_times, "gc-times");
 SCM_SYMBOL (sym_cells_marked, "cells-marked");
+SCM_SYMBOL (sym_cells_marked_conservatively, "cells-marked-conservatively");
 SCM_SYMBOL (sym_cells_swept, "cells-swept");
 SCM_SYMBOL (sym_malloc_yield, "malloc-yield");
 SCM_SYMBOL (sym_cell_yield, "cell-yield");
@@ -243,8 +250,6 @@ SCM_SYMBOL (sym_protected_objects, "protected-objects");
 SCM_SYMBOL (sym_total_cells_allocated, "total-cells-allocated");
 
 
-
-
 /* Number of calls to SCM_NEWCELL since startup.  */
 unsigned scm_newcell_count;
 unsigned scm_newcell2_count;
@@ -313,57 +318,51 @@ SCM_DEFINE (scm_gc_stats, "gc-stats", 0, 0, 0,
   unsigned long int local_protected_obj_count;
   double local_scm_gc_cells_swept;
   double local_scm_gc_cells_marked;
+  double local_scm_gc_cells_marked_conservatively;
   double local_scm_total_cells_allocated;
   SCM answer;
   unsigned long *bounds = 0;
-  int table_size = scm_i_heap_segment_table_size;  
+  int table_size = 0;
   SCM_CRITICAL_SECTION_START;
 
-  /*
-    temporarily store the numbers, so as not to cause GC.
-   */
- 
-  bounds = malloc (sizeof (unsigned long)  * table_size * 2);
-  if (!bounds)
-    abort();
-  for (i = table_size; i--; )
-    {
-      bounds[2*i] = (unsigned long)scm_i_heap_segment_table[i]->bounds[0];
-      bounds[2*i+1] = (unsigned long)scm_i_heap_segment_table[i]->bounds[1];
-    }
-
+  bounds = scm_i_segment_table_info (&table_size);
 
   /* Below, we cons to produce the resulting list.  We want a snapshot of
    * the heap situation before consing.
    */
   local_scm_mtrigger = scm_mtrigger;
   local_scm_mallocated = scm_mallocated;
-  local_scm_heap_size = SCM_HEAP_SIZE;
+  local_scm_heap_size =
+    (scm_i_master_freelist.heap_total_cells + scm_i_master_freelist2.heap_total_cells);
 
-  local_scm_cells_allocated = scm_cells_allocated;
+  local_scm_cells_allocated =
+    scm_cells_allocated + scm_i_gc_sweep_stats.collected;
   
   local_scm_gc_time_taken = scm_gc_time_taken;
   local_scm_gc_mark_time_taken = scm_gc_mark_time_taken;
   local_scm_gc_times = scm_gc_times;
   local_scm_gc_malloc_yield_percentage = scm_gc_malloc_yield_percentage;
-  local_scm_gc_cell_yield_percentage=  scm_gc_cell_yield_percentage;
+  local_scm_gc_cell_yield_percentage = scm_gc_cell_yield_percentage;
   local_protected_obj_count = protected_obj_count;
   local_scm_gc_cells_swept =
     (double) scm_gc_cells_swept_acc
-    + (double) scm_gc_cells_swept;
+    + (double) scm_i_gc_sweep_stats.swept;
   local_scm_gc_cells_marked = scm_gc_cells_marked_acc 
-    +(double) scm_gc_cells_swept 
-    -(double) scm_gc_cells_collected;
+    + (double) scm_i_gc_sweep_stats.swept
+    - (double) scm_i_gc_sweep_stats.collected;
+  local_scm_gc_cells_marked_conservatively
+    = scm_gc_cells_marked_conservatively_acc;
 
   local_scm_total_cells_allocated = scm_gc_cells_allocated_acc
-    + (double) (scm_cells_allocated - scm_last_cells_allocated);
-
+    + (double) scm_i_gc_sweep_stats.collected;
+  
   for (i = table_size; i--;)
     {
       heap_segs = scm_cons (scm_cons (scm_from_ulong (bounds[2*i]),
 				      scm_from_ulong (bounds[2*i+1])),
 			    heap_segs);
     }
+  
   /* njrev: can any of these scm_cons's or scm_list_n signal a memory
      error?  If so we need a frame here. */
   answer =
@@ -372,9 +371,11 @@ SCM_DEFINE (scm_gc_stats, "gc-stats", 0, 0, 0,
 		scm_cons (sym_cells_allocated,
 			  scm_from_ulong (local_scm_cells_allocated)),
 		scm_cons (sym_total_cells_allocated,
-			  scm_from_ulong (local_scm_total_cells_allocated)),
+			  scm_from_double (local_scm_total_cells_allocated)),
 		scm_cons (sym_heap_size,
 			  scm_from_ulong (local_scm_heap_size)),
+		scm_cons (sym_cells_marked_conservatively,
+			  scm_from_ulong (local_scm_gc_cells_marked_conservatively)),
 		scm_cons (sym_mallocated,
 			  scm_from_ulong (local_scm_mallocated)),
 		scm_cons (sym_mtrigger,
@@ -388,7 +389,7 @@ SCM_DEFINE (scm_gc_stats, "gc-stats", 0, 0, 0,
 		scm_cons (sym_cells_swept,
 			  scm_from_double (local_scm_gc_cells_swept)),
 		scm_cons (sym_malloc_yield,
-			  scm_from_long(local_scm_gc_malloc_yield_percentage)),
+			  scm_from_long (local_scm_gc_malloc_yield_percentage)),
 		scm_cons (sym_cell_yield,
 			  scm_from_long (local_scm_gc_cell_yield_percentage)),
 		scm_cons (sym_protected_objects,
@@ -402,40 +403,27 @@ SCM_DEFINE (scm_gc_stats, "gc-stats", 0, 0, 0,
 }
 #undef FUNC_NAME
 
-static void
-gc_start_stats (const char *what SCM_UNUSED)
-{
-  t_before_gc = scm_c_get_internal_run_time ();
-
-  scm_gc_cells_marked_acc += (double) scm_gc_cells_swept
-    - (double) scm_gc_cells_collected;
-  scm_gc_cells_swept_acc += (double) scm_gc_cells_swept;
-
-  scm_gc_cell_yield_percentage = ( scm_gc_cells_collected * 100 ) / SCM_HEAP_SIZE; 
-  
-  scm_gc_cells_swept = 0;
-  scm_gc_cells_collected_1 = scm_gc_cells_collected;
-
-  /*
-    CELLS SWEPT is another word for the number of cells that were
-    examined during GC. YIELD is the number that we cleaned
-    out. MARKED is the number that weren't cleaned. 
-   */
-  scm_gc_cells_collected = 0;
-  scm_gc_malloc_collected = 0;
-  scm_gc_ports_collected = 0;
-}
-
+/*
+  Update nice-to-know-statistics.
+ */
 static void
 gc_end_stats ()
 {
-  unsigned long t = scm_c_get_internal_run_time ();
-  scm_gc_time_taken += (t - t_before_gc);
+  /* CELLS SWEPT is another word for the number of cells that were examined
+     during GC. YIELD is the number that we cleaned out. MARKED is the number
+     that weren't cleaned.  */
+  scm_gc_cell_yield_percentage = (scm_i_gc_sweep_stats.collected * 100) /
+    (scm_i_master_freelist.heap_total_cells + scm_i_master_freelist2.heap_total_cells);
+
+  scm_gc_cells_allocated_acc +=
+    (double) scm_i_gc_sweep_stats.collected;
+  scm_gc_cells_marked_acc += (double) scm_i_last_marked_cell_count;
+  scm_gc_cells_marked_conservatively_acc += (double) scm_i_find_heap_calls;
+  scm_gc_cells_swept_acc += (double) scm_i_gc_sweep_stats.swept;
 
   ++scm_gc_times;
 }
 
-
 SCM_DEFINE (scm_object_address, "object-address", 1, 0, 0,
             (SCM obj),
 	    "Return an integer that for the lifetime of @var{obj} is uniquely\n"
@@ -482,42 +470,50 @@ scm_gc_for_newcell (scm_t_cell_type_statistics *freelist, SCM *free_cells)
 {
   SCM cell;
   int did_gc = 0;
- 
+
   scm_i_scm_pthread_mutex_lock (&scm_i_sweep_mutex);
   scm_gc_running_p = 1;
-
-  *free_cells = scm_i_sweep_some_segments (freelist);
-  if (*free_cells == SCM_EOL && scm_i_gc_grow_heap_p (freelist))
+  
+  *free_cells = scm_i_sweep_for_freelist (freelist);
+  if (*free_cells == SCM_EOL)
     {
-      freelist->heap_segment_idx = scm_i_get_new_heap_segment (freelist, abort_on_error);
-      *free_cells = scm_i_sweep_some_segments (freelist);
-    }
+      float delta = scm_i_gc_heap_size_delta (freelist);
+      if (delta > 0.0)
+	{
+	  size_t bytes = ((unsigned long) delta) * sizeof (scm_t_cell);
+	  freelist->heap_segment_idx =
+	    scm_i_get_new_heap_segment (freelist, bytes, abort_on_error);
 
+	  *free_cells = scm_i_sweep_for_freelist (freelist);
+	}
+    }
+  
   if (*free_cells == SCM_EOL)
     {
-      /*
-	with the advent of lazy sweep, GC yield is only known just
-	before doing the GC.
-      */
-      scm_i_adjust_min_yield (freelist);
-
       /*
 	out of fresh cells. Try to get some new ones.
        */
-
+      char reason[] = "0-cells";
+      reason[0] += freelist->span;
+      
       did_gc = 1;
-      scm_i_gc ("cells");
+      scm_i_gc (reason);
 
-      *free_cells = scm_i_sweep_some_segments (freelist);
+      *free_cells = scm_i_sweep_for_freelist (freelist);
     }
   
   if (*free_cells == SCM_EOL)
     {
       /*
 	failed getting new cells. Get new juice or die.
-       */
-      freelist->heap_segment_idx = scm_i_get_new_heap_segment (freelist, abort_on_error);
-      *free_cells = scm_i_sweep_some_segments (freelist);
+      */
+      float delta = scm_i_gc_heap_size_delta (freelist);
+      assert (delta > 0.0);
+      size_t bytes = ((unsigned long) delta) * sizeof (scm_t_cell);
+      freelist->heap_segment_idx =
+	scm_i_get_new_heap_segment (freelist, bytes, abort_on_error);
+
+      *free_cells = scm_i_sweep_for_freelist (freelist);
     }
   
   if (*free_cells == SCM_EOL)
@@ -543,39 +539,9 @@ scm_t_c_hook scm_before_sweep_c_hook;
 scm_t_c_hook scm_after_sweep_c_hook;
 scm_t_c_hook scm_after_gc_c_hook;
 
-/* Must be called while holding scm_i_sweep_mutex.
- */
-
-void
-scm_i_gc (const char *what)
+static void
+scm_check_deprecated_memory_return ()
 {
-  scm_i_thread_put_to_sleep ();
-
-  scm_c_hook_run (&scm_before_gc_c_hook, 0);
-
-#ifdef DEBUGINFO
-  fprintf (stderr,"gc reason %s\n", what);
-  
-  fprintf (stderr,
-	   scm_is_null (*SCM_FREELIST_LOC (scm_i_freelist))
-	   ? "*"
-	   : (scm_is_null (*SCM_FREELIST_LOC (scm_i_freelist2)) ? "o" : "m"));
-#endif
-
-  gc_start_stats (what);
-
-  /*
-    Set freelists to NULL so scm_cons() always triggers gc, causing
-    the assertion above to fail.
-  */
-  *SCM_FREELIST_LOC (scm_i_freelist) = SCM_EOL;
-  *SCM_FREELIST_LOC (scm_i_freelist2) = SCM_EOL;
-  
-  /*
-    Let's finish the sweep. The conservative GC might point into the
-    garbage, and marking that would create a mess.
-   */
-  scm_i_sweep_all_segments("GC");
   if (scm_mallocated < scm_i_deprecated_memory_return)
     {
       /* The byte count of allocated objects has underflowed.  This is
@@ -590,14 +556,78 @@ scm_i_gc (const char *what)
       abort ();
     }
   scm_mallocated -= scm_i_deprecated_memory_return;
+  scm_i_deprecated_memory_return = 0;
+}
 
+long int scm_i_last_marked_cell_count;
+
+/* Must be called while holding scm_i_sweep_mutex.
+
+   This function is fairly long, but it touches various global
+   variables. To not obscure the side effects on global variables,
+   this function has not been split up.
+ */
+void
+scm_i_gc (const char *what)
+{
+  unsigned long t_before_gc = 0;
+  
+  scm_i_thread_put_to_sleep ();
+  
+  scm_c_hook_run (&scm_before_gc_c_hook, 0);
+
+#ifdef DEBUGINFO
+  fprintf (stderr,"gc reason %s\n", what);
+  fprintf (stderr,
+	   scm_is_null (*SCM_FREELIST_LOC (scm_i_freelist))
+	   ? "*"
+	   : (scm_is_null (*SCM_FREELIST_LOC (scm_i_freelist2)) ? "o" : "m"));
+#endif
+
+  t_before_gc = scm_c_get_internal_run_time ();
+  scm_gc_malloc_collected = 0;
+
+  /*
+    Set freelists to NULL so scm_cons () always triggers gc, causing
+    the assertion above to fail.
+  */
+  *SCM_FREELIST_LOC (scm_i_freelist) = SCM_EOL;
+  *SCM_FREELIST_LOC (scm_i_freelist2) = SCM_EOL;
+  
+  /*
+    Let's finish the sweep. The conservative GC might point into the
+    garbage, and marking that would create a mess.
+   */
+  scm_i_sweep_all_segments ("GC", &scm_i_gc_sweep_stats);
+  scm_check_deprecated_memory_return ();
+
+#if (SCM_DEBUG_CELL_ACCESSES == 0 && SCM_SIZEOF_UNSIGNED_LONG == 4)
+  /* Sanity check our numbers. */
+  /* TODO(hanwen): figure out why the stats are off on x64_64. */
+  /* If this was not true, someone touched mark bits outside of the
+     mark phase. */
+  if (scm_i_last_marked_cell_count != scm_i_marked_count ())
+    {
+      static char msg[] =
+	"The number of marked objects changed since the last GC: %d vs %d.";
+      /* At some point, we should probably use a deprecation warning. */
+      fprintf(stderr, msg, scm_i_last_marked_cell_count, scm_i_marked_count ());
+    }
+  assert (scm_i_gc_sweep_stats.swept
+	  == (scm_i_master_freelist.heap_total_cells
+	      + scm_i_master_freelist2.heap_total_cells));
+  assert (scm_i_gc_sweep_stats.collected + scm_i_last_marked_cell_count
+	  == scm_i_gc_sweep_stats.swept);
+#endif /* SCM_DEBUG_CELL_ACCESSES */
   
   /* Mark */
-
   scm_c_hook_run (&scm_before_mark_c_hook, 0);
+
   scm_mark_all ();
   scm_gc_mark_time_taken += (scm_c_get_internal_run_time () - t_before_gc);
 
+  scm_i_last_marked_cell_count = scm_cells_allocated = scm_i_marked_count ();
+
   /* Sweep
 
     TODO: the after_sweep hook should probably be moved to just before
@@ -623,22 +653,41 @@ scm_i_gc (const char *what)
     distinct classes of hook functions since this can prevent some
     bad interference when several modules adds gc hooks.
    */
-
   scm_c_hook_run (&scm_before_sweep_c_hook, 0);
-  scm_gc_sweep ();
+
+  /*
+    Nothing here: lazy sweeping.
+   */
+  scm_i_reset_segments ();
+  
+  *SCM_FREELIST_LOC (scm_i_freelist) = SCM_EOL;
+  *SCM_FREELIST_LOC (scm_i_freelist2) = SCM_EOL;
+
+  /* Invalidate the freelists of other threads. */
+  scm_i_thread_invalidate_freelists ();
+
   scm_c_hook_run (&scm_after_sweep_c_hook, 0);
 
   gc_end_stats ();
 
+  scm_i_gc_sweep_stats.collected = scm_i_gc_sweep_stats.swept = 0;
+  scm_i_gc_sweep_freelist_reset (&scm_i_master_freelist);
+  scm_i_gc_sweep_freelist_reset (&scm_i_master_freelist2);
+  
+  /* Arguably, this statistic is fairly useless: marking will dominate
+     the time taken.
+  */
+  scm_gc_time_taken += (scm_c_get_internal_run_time () - t_before_gc);
+    
   scm_i_thread_wake_up ();
-
   /*
     For debugging purposes, you could do
-    scm_i_sweep_all_segments("debug"), but then the remains of the
+    scm_i_sweep_all_segments ("debug"), but then the remains of the
     cell aren't left to analyse.
    */
 }
 
+
 
 /* {GC Protection Helper Functions}
  */
@@ -737,7 +786,7 @@ scm_permanent_object (SCM obj)
 */
 
 /* Implementation note:  For every object X, there is a counter which
-   scm_gc_protect_object(X) increments and scm_gc_unprotect_object(X) decrements.
+   scm_gc_protect_object (X) increments and scm_gc_unprotect_object (X) decrements.
 */
 
 
@@ -912,18 +961,9 @@ scm_init_storage ()
   while (j)
     scm_sys_protects[--j] = SCM_BOOL_F;
 
-  scm_gc_init_freelist();
+  scm_gc_init_freelist ();
   scm_gc_init_malloc ();
 
-  j = SCM_HEAP_SEG_SIZE;
-
-  
-  /* Initialise the list of ports.  */
-  scm_i_port_table = (scm_t_port **)
-    malloc (sizeof (scm_t_port *) * scm_i_port_table_room);
-  if (!scm_i_port_table)
-    return 1;
-
 #if 0
   /* We can't have a cleanup handler since we have no thread to run it
      in. */
@@ -971,7 +1011,7 @@ gc_async_thunk (void)
  */
 static void *
 mark_gc_async (void * hook_data SCM_UNUSED,
-	       void *func_data SCM_UNUSED,
+	       void *fn_data SCM_UNUSED,
 	       void *data SCM_UNUSED)
 {
   /* If cell access debugging is enabled, the user may choose to perform
@@ -1043,7 +1083,7 @@ void *
 scm_ia64_ar_bsp (const void *ctx)
 {
   uint64_t bsp;
-  __uc_get_ar_bsp(ctx, &bsp);
+  __uc_get_ar_bsp (ctx, &bsp);
   return (void *) bsp;
 }
 # endif /* hpux */
@@ -1058,7 +1098,7 @@ scm_ia64_register_backing_store_base (void)
 void *
 scm_ia64_ar_bsp (const void *opaque)
 {
-  ucontext_t *ctx = opaque;
+  const ucontext_t *ctx = opaque;
   return (void *) ctx->uc_mcontext.sc_ar_bsp;
 }
 # endif	/* linux */
@@ -1068,21 +1108,6 @@ void
 scm_gc_sweep (void)
 #define FUNC_NAME "scm_gc_sweep"
 {
-  scm_i_deprecated_memory_return = 0;
-
-  scm_i_gc_sweep_freelist_reset (&scm_i_master_freelist);
-  scm_i_gc_sweep_freelist_reset (&scm_i_master_freelist2);
-
-  /*
-    NOTHING HERE: LAZY SWEEPING ! 
-   */
-  scm_i_reset_segments ();
-  
-  *SCM_FREELIST_LOC (scm_i_freelist) = SCM_EOL;
-  *SCM_FREELIST_LOC (scm_i_freelist2) = SCM_EOL;
-
-  /* Invalidate the freelists of other threads. */
-  scm_i_thread_invalidate_freelists ();
 }
 
 #undef FUNC_NAME
