@@ -6,18 +6,19 @@
 /* Copyright (C) 2001, 2002, 2003, 2004, 2006, 2008 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 /* This file is for inline functions.  On platforms that don't support
@@ -54,7 +55,7 @@
    C99 mode and doesn't define `__GNUC_STDC_INLINE__'.  Fall back to "static
    inline" in that case.  */
 
-# if (defined __GNUC__) && (!(__APPLE_CC__ > 5400 && __STDC_VERSION__ >= 199901L))
+# if (defined __GNUC__) && (!(((defined __APPLE_CC__) && (__APPLE_CC__ > 5400)) && __STDC_VERSION__ >= 199901L))
 #  define SCM_C_USE_EXTERN_INLINE 1
 #  if (defined __GNUC_STDC_INLINE__) || (__GNUC__ == 4 && __GNUC_MINOR__ == 2)
 #   define SCM_C_EXTERN_INLINE					\
@@ -119,13 +120,6 @@ scm_cell (scm_t_bits car, scm_t_bits cdr)
       *freelist = SCM_FREE_CELL_CDR (*freelist);
     }
 
-  /*
-    We update scm_cells_allocated from this function. If we don't
-    update this explicitly, we will have to walk a freelist somewhere
-    later on, which seems a lot more expensive.
-   */
-  scm_cells_allocated += 1;  
-
 #if (SCM_DEBUG_CELL_ACCESSES == 1)
     if (scm_debug_cell_accesses_p)
       {
@@ -141,18 +135,19 @@ scm_cell (scm_t_bits car, scm_t_bits cdr)
 	  }
       }
 
+#if (SCM_DEBUG_MARKING_API == 0)
     /*
       Always set mark. Otherwise cells that are alloced before
       scm_debug_cell_accesses_p is toggled seem invalid.
     */
     SCM_SET_GC_MARK (z);
-
+#endif /* SCM_DEBUG_MARKING_API */
+    
     /*
       TODO: figure out if this use of mark bits is valid with
       threading. What if another thread is doing GC at this point
       ... ?
      */
-      
 #endif
 
   
@@ -190,8 +185,6 @@ scm_double_cell (scm_t_bits car, scm_t_bits cbr,
       *freelist = SCM_FREE_CELL_CDR (*freelist);
     }
 
-  scm_cells_allocated += 2;
-
   /* Initialize the type slot last so that the cell is ignored by the
      GC until it is completely initialized.  This is only relevant
      when the GC can actually run during this code, which it can't
@@ -212,10 +205,11 @@ scm_double_cell (scm_t_bits car, scm_t_bits cbr,
 	  abort();
 	}
     }
-
+#if (SCM_DEBUG_MARKING_API == 0)
   /* see above. */
   SCM_SET_GC_MARK (z);
-
+#endif /* SCM_DEBUG_MARKING_API */
+  
 #endif
 
   /* When this function is inlined, it's possible that the last
