@@ -1,22 +1,26 @@
-/* Copyright (C) 1995-1999,2000,2001, 2002, 2003, 2004, 2006 Free Software Foundation, Inc.
+/* Copyright (C) 1995-1999,2000,2001, 2002, 2003, 2004, 2006, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <errno.h>
 
@@ -43,6 +47,9 @@
 
 #include "libguile/validate.h"
 #include "libguile/print.h"
+
+#include "libguile/private-options.h"
+
 
 
 /* {Names of immediate symbols}
@@ -83,7 +90,9 @@ scm_t_option scm_print_opts[] = {
     "How to print symbols that have a colon as their first or last character. "
     "The value '#f' does not quote the colons; '#t' quotes them; "
     "'reader' quotes them when the reader option 'keywords' is not '#f'." 
-  }
+  },
+  { 0 },
+  
 };
 
 SCM_DEFINE (scm_print_options, "print-options-interface", 0, 1, 0, 
@@ -96,7 +105,6 @@ SCM_DEFINE (scm_print_options, "print-options-interface", 0, 1, 0,
 {
   SCM ans = scm_options (setting,
 			 scm_print_opts,
-			 SCM_N_PRINT_OPTIONS,
 			 FUNC_NAME);
   return ans;
 }
@@ -647,33 +655,10 @@ iprin1 (SCM exp, SCM port, scm_print_state *pstate)
 		    ? "#<primitive-generic "
 		    : "#<primitive-procedure ",
 		    port);
-	  scm_puts (scm_i_symbol_chars (SCM_SNAME (exp)), port);
+	  scm_puts (scm_i_symbol_chars (SCM_SUBR_NAME (exp)), port);
 	  scm_putc ('>', port);
 	  break;
-#ifdef CCLO
-	case scm_tc7_cclo:
-	  {
-	    SCM proc = SCM_CCLO_SUBR (exp);
-	    if (scm_is_eq (proc, scm_f_gsubr_apply))
-	      {
-		/* Print gsubrs as primitives */
-		SCM name = scm_procedure_name (exp);
-		scm_puts ("#<primitive-procedure", port);
-		if (scm_is_true (name))
-		  {
-		    scm_putc (' ', port);
-		    scm_puts (scm_i_symbol_chars (name), port);
-		  }
-	      }
-	    else
-	      {
-		scm_puts ("#<compiled-closure ", port);
-		scm_iprin1 (proc, port, pstate);
-	      }
-	    scm_putc ('>', port);
-	  }
-	  break;
-#endif
+
 	case scm_tc7_pws:
 	  scm_puts ("#<procedure-with-setter", port);
 	  {
@@ -1165,7 +1150,7 @@ scm_init_print ()
 {
   SCM vtable, layout, type;
 
-  scm_init_opts (scm_print_options, scm_print_opts, SCM_N_PRINT_OPTIONS);
+  scm_init_opts (scm_print_options, scm_print_opts);
 
   scm_print_options (scm_list_4 (scm_from_locale_symbol ("highlight-prefix"),
 				 scm_from_locale_string ("{"),
