@@ -1,22 +1,26 @@
-/* Copyright (C) 1995,1996,1997,1999,2000,2001, 2006 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1999,2000,2001, 2006, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include "libguile/_scm.h"
 
@@ -28,46 +32,33 @@
 
 #include "libguile/validate.h"
 #include "libguile/procs.h"
+#include "libguile/procprop.h"
+#include "libguile/objcodes.h"
+#include "libguile/programs.h"
 
 
 
 /* {Procedures}
  */
 
-scm_t_subr_entry *scm_subr_table;
 
-/* libguile contained approx. 700 primitive procedures on 24 Aug 1999. */
-
-/* Increased to 800 on 2001-05-07 -- Guile now has 779 primitives on
-   startup, 786 with guile-readline.  'martin */
-
-long scm_subr_table_size = 0;
-long scm_subr_table_room = 800;
-
-SCM 
+SCM
 scm_c_make_subr (const char *name, long type, SCM (*fcn) ())
 {
   register SCM z;
-  long entry;
+  SCM sname;
+  SCM *meta_info;
 
-  if (scm_subr_table_size == scm_subr_table_room)
-    {
-      long new_size = scm_subr_table_room * 3 / 2;
-      void *new_table
-	= scm_realloc ((char *) scm_subr_table,
-		       sizeof (scm_t_subr_entry) * new_size);
-      scm_subr_table = new_table;
-      scm_subr_table_room = new_size;
-    }
+  meta_info = scm_gc_malloc (2 * sizeof (*meta_info), "subr meta-info");
+  sname = scm_from_locale_symbol (name);
+  meta_info[0] = sname;
+  meta_info[1] = SCM_EOL;  /* properties */
+
+  z = scm_double_cell ((scm_t_bits) type, (scm_t_bits) fcn,
+		       0 /* generic */, (scm_t_bits) meta_info);
+
+  scm_remember_upto_here_1 (sname);
 
-  entry = scm_subr_table_size;
-  z = scm_cell ((entry << 8) + type, (scm_t_bits) fcn);
-  scm_subr_table[entry].handle = z;
-  scm_subr_table[entry].name = scm_from_locale_symbol (name);
-  scm_subr_table[entry].generic = 0;
-  scm_subr_table[entry].properties = SCM_EOL;
-  scm_subr_table_size++;
-  
   return z;
 }
 
@@ -75,7 +66,7 @@ SCM
 scm_c_define_subr (const char *name, long type, SCM (*fcn) ())
 {
   SCM subr = scm_c_make_subr (name, type, fcn);
-  scm_define (SCM_SUBR_ENTRY(subr).name, subr);
+  scm_define (SCM_SUBR_NAME (subr), subr);
   return subr;
 }
 
@@ -84,11 +75,8 @@ scm_c_define_subr (const char *name, long type, SCM (*fcn) ())
 void
 scm_free_subr_entry (SCM subr)
 {
-  long entry = SCM_SUBRNUM (subr);
-  /* Move last entry in table to the free position */
-  scm_subr_table[entry] = scm_subr_table[scm_subr_table_size - 1];
-  SCM_SET_SUBRNUM (scm_subr_table[entry].handle, entry);
-  scm_subr_table_size--;
+  scm_gc_free (SCM_SUBR_META_INFO (subr), 2 * sizeof (SCM),
+	       "subr meta-info");
 }
 
 SCM
@@ -96,7 +84,7 @@ scm_c_make_subr_with_generic (const char *name,
 			      long type, SCM (*fcn) (), SCM *gf)
 {
   SCM subr = scm_c_make_subr (name, type, fcn);
-  SCM_SUBR_ENTRY(subr).generic = gf;
+  SCM_SET_SUBR_GENERIC_LOC (subr, gf);
   return subr;
 }
 
@@ -105,57 +93,10 @@ scm_c_define_subr_with_generic (const char *name,
 				long type, SCM (*fcn) (), SCM *gf)
 {
   SCM subr = scm_c_make_subr_with_generic (name, type, fcn, gf);
-  scm_define (SCM_SUBR_ENTRY(subr).name, subr);
+  scm_define (SCM_SUBR_NAME (subr), subr);
   return subr;
 }
 
-void
-scm_mark_subr_table ()
-{
-  long i;
-  for (i = 0; i < scm_subr_table_size; ++i)
-    {
-      scm_gc_mark (scm_subr_table[i].name);
-      if (scm_subr_table[i].generic && *scm_subr_table[i].generic)
-	scm_gc_mark (*scm_subr_table[i].generic);
-      if (SCM_NIMP (scm_subr_table[i].properties))
-	scm_gc_mark (scm_subr_table[i].properties);
-    }
-}
-
-
-#ifdef CCLO
-SCM 
-scm_makcclo (SCM proc, size_t len)
-{
-  scm_t_bits *base = scm_gc_malloc (len * sizeof (scm_t_bits),
-				    "compiled closure");
-  unsigned long i;
-  SCM s;
-
-  for (i = 0; i < len; ++i)
-    base [i] = SCM_UNPACK (SCM_UNSPECIFIED);
-
-  s = scm_cell (SCM_MAKE_CCLO_TAG (len), (scm_t_bits) base);
-  SCM_SET_CCLO_SUBR (s, proc);
-  return s;
-}
-
-/* Undocumented debugging procedure */
-#ifdef GUILE_DEBUG
-SCM_DEFINE (scm_make_cclo, "make-cclo", 2, 0, 0,
-            (SCM proc, SCM len),
-	    "Create a compiled closure for @var{proc}, which reserves\n"
-	    "@var{len} objects for its usage.")
-#define FUNC_NAME s_scm_make_cclo
-{
-  return scm_makcclo (proc, scm_to_size_t (len));
-}
-#undef FUNC_NAME
-#endif
-#endif
-
-
 
 SCM_DEFINE (scm_procedure_p, "procedure?", 1, 0, 0, 
 	    (SCM obj),
@@ -170,9 +111,6 @@ SCM_DEFINE (scm_procedure_p, "procedure?", 1, 0, 0,
 	  break;
       case scm_tcs_closures:
       case scm_tcs_subrs:
-#ifdef CCLO
-      case scm_tc7_cclo:
-#endif
       case scm_tc7_pws:
 	return SCM_BOOL_T;
       case scm_tc7_smob:
@@ -210,15 +148,16 @@ SCM_DEFINE (scm_thunk_p, "thunk?", 1, 0, 0,
 	case scm_tc7_lsubr:
 	case scm_tc7_rpsubr:
 	case scm_tc7_asubr:
-#ifdef CCLO
-	case scm_tc7_cclo:
-#endif
 	  return SCM_BOOL_T;
+	case scm_tc7_gsubr:
+	  return scm_from_bool (SCM_GSUBR_REQ (SCM_GSUBR_TYPE (obj)) == 0);
 	case scm_tc7_pws:
 	  obj = SCM_PROCEDURE (obj);
 	  goto again;
 	default:
-	  ;
+          if (SCM_PROGRAM_P (obj) && SCM_PROGRAM_DATA (obj)->nargs == 0)
+            return SCM_BOOL_T;
+          /* otherwise fall through */
 	}
     }
   return SCM_BOOL_F;
@@ -264,12 +203,6 @@ SCM_DEFINE (scm_procedure_documentation, "procedure-documentation", 1, 0, 0,
 	return SCM_BOOL_F;
     default:
       return SCM_BOOL_F;
-/*
-  case scm_tcs_subrs:
-#ifdef CCLO
-  case scm_tc7_cclo:
-#endif
-*/
     }
 }
 #undef FUNC_NAME
@@ -294,11 +227,25 @@ SCM_DEFINE (scm_make_procedure_with_setter, "make-procedure-with-setter", 2, 0,
 	    "with the associated setter @var{setter}.")
 #define FUNC_NAME s_scm_make_procedure_with_setter
 {
+  SCM name, ret;
   SCM_VALIDATE_PROC (1, procedure);
   SCM_VALIDATE_PROC (2, setter);
-  return scm_double_cell (scm_tc7_pws,
-			  SCM_UNPACK (procedure),
-			  SCM_UNPACK (setter), 0);
+  ret = scm_double_cell (scm_tc7_pws,
+                         SCM_UNPACK (procedure),
+                         SCM_UNPACK (setter), 0);
+  /* don't use procedure_name, because don't care enough to do a reverse
+     lookup */
+  switch (SCM_TYP7 (procedure)) {
+  case scm_tcs_subrs:
+    name = SCM_SUBR_NAME (procedure);
+    break;
+  default:
+    name = scm_procedure_property (procedure, scm_sym_name);
+    break;
+  }
+  if (scm_is_true (name))
+    scm_set_procedure_property_x (ret, scm_sym_name, name);
+  return ret;
 }
 #undef FUNC_NAME
 
@@ -345,15 +292,7 @@ scm_setter (SCM proc)
   return SCM_BOOL_F; /* not reached */
 }
 
-
-void
-scm_init_subr_table ()
-{
-  scm_subr_table
-    = ((scm_t_subr_entry *)
-       scm_malloc (sizeof (scm_t_subr_entry) * scm_subr_table_room));
-}
-
+
 void
 scm_init_procs ()
 {
