@@ -1,18 +1,19 @@
-/* Copyright (C) 1996,1998,2000,2001,2004 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1998,2000,2001,2004,2005, 2006, 2008 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
@@ -24,6 +25,9 @@
 
 
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include "libguile/_scm.h"
 #include "libguile/strings.h"
@@ -308,10 +312,10 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
 	plvra = SCM_CDRLOC (*plvra);
       }
 
-    scm_frame_begin (0);
+    scm_dynwind_begin (0);
 
     vinds = scm_malloc (sizeof(long) * SCM_I_ARRAY_NDIM (ra0));
-    scm_frame_free (vinds);
+    scm_dynwind_free (vinds);
 
     for (k = 0; k <= kmax; k++)
       vinds[k] = SCM_I_ARRAY_DIMS (ra0)[k].lbnd;
@@ -340,7 +344,7 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
       }
     while (k >= 0);
 
-    scm_frame_end ();
+    scm_dynwind_end ();
     return 1;
     }
 }
@@ -420,22 +424,30 @@ int
 scm_ra_eqp (SCM ra0, SCM ras)
 {
   SCM ra1 = SCM_CAR (ras), ra2 = SCM_CAR (SCM_CDR (ras));
-  long n = SCM_I_ARRAY_DIMS (ra0)->ubnd - SCM_I_ARRAY_DIMS (ra0)->lbnd + 1;
-  unsigned long i0 = SCM_I_ARRAY_BASE (ra0), i1 = SCM_I_ARRAY_BASE (ra1), i2 = SCM_I_ARRAY_BASE (ra2);
-  long inc0 = SCM_I_ARRAY_DIMS (ra0)->inc;
+  scm_t_array_handle ra0_handle;
+  scm_t_array_dim *ra0_dims;
+  size_t n;
+  ssize_t inc0;
+  size_t i0 = 0;
+  unsigned long i1 = SCM_I_ARRAY_BASE (ra1), i2 = SCM_I_ARRAY_BASE (ra2);
   long inc1 = SCM_I_ARRAY_DIMS (ra1)->inc;
   long inc2 = SCM_I_ARRAY_DIMS (ra1)->inc;
-  ra0 = SCM_I_ARRAY_V (ra0);
   ra1 = SCM_I_ARRAY_V (ra1);
   ra2 = SCM_I_ARRAY_V (ra2);
-  
+
+  scm_array_get_handle (ra0, &ra0_handle);
+  ra0_dims = scm_array_handle_dims (&ra0_handle);
+  n = ra0_dims[0].ubnd - ra0_dims[0].lbnd + 1;
+  inc0 = ra0_dims[0].inc;
+
   {
     for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-      if (scm_is_true (scm_c_bitvector_ref (ra0, i0)))
+      if (scm_is_true (scm_array_handle_ref (&ra0_handle, i0)))
 	if (!scm_is_eq (GVREF (ra1, i1), GVREF (ra2, i2)))
-	  scm_c_bitvector_set_x (ra0, i0, SCM_BOOL_F);
+	  scm_array_handle_set (&ra0_handle, i0, SCM_BOOL_F);
   }
 
+  scm_array_handle_release (&ra0_handle);
   return 1;
 }
 
@@ -444,24 +456,32 @@ scm_ra_eqp (SCM ra0, SCM ras)
 static int
 ra_compare (SCM ra0, SCM ra1, SCM ra2, int opt)
 {
-  long n = SCM_I_ARRAY_DIMS (ra0)->ubnd - SCM_I_ARRAY_DIMS (ra0)->lbnd + 1;
-  unsigned long i0 = SCM_I_ARRAY_BASE (ra0), i1 = SCM_I_ARRAY_BASE (ra1), i2 = SCM_I_ARRAY_BASE (ra2);
-  long inc0 = SCM_I_ARRAY_DIMS (ra0)->inc;
+  scm_t_array_handle ra0_handle;
+  scm_t_array_dim *ra0_dims;
+  size_t n;
+  ssize_t inc0;
+  size_t i0 = 0;
+  unsigned long i1 = SCM_I_ARRAY_BASE (ra1), i2 = SCM_I_ARRAY_BASE (ra2);
   long inc1 = SCM_I_ARRAY_DIMS (ra1)->inc;
   long inc2 = SCM_I_ARRAY_DIMS (ra1)->inc;
-  ra0 = SCM_I_ARRAY_V (ra0);
   ra1 = SCM_I_ARRAY_V (ra1);
   ra2 = SCM_I_ARRAY_V (ra2);
 
+  scm_array_get_handle (ra0, &ra0_handle);
+  ra0_dims = scm_array_handle_dims (&ra0_handle);
+  n = ra0_dims[0].ubnd - ra0_dims[0].lbnd + 1;
+  inc0 = ra0_dims[0].inc;
+
   {
     for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-      if (scm_is_true (scm_c_bitvector_ref (ra0, i0)))
+      if (scm_is_true (scm_array_handle_ref (&ra0_handle, i0)))
 	if (opt ?
 	    scm_is_true (scm_less_p (GVREF (ra1, i1), GVREF (ra2, i2))) :
 	    scm_is_false (scm_less_p (GVREF (ra1, i1), GVREF (ra2, i2))))
-	  scm_c_bitvector_set_x (ra0, i0, SCM_BOOL_F);
+	  scm_array_handle_set (&ra0_handle, i0, SCM_BOOL_F);
   }
 
+  scm_array_handle_release (&ra0_handle);
   return 1;
 }
 
@@ -797,6 +817,7 @@ ramap_a (SCM ra0, SCM proc, SCM ras)
 
 SCM_REGISTER_PROC(s_array_map_in_order_x, "array-map-in-order!", 2, 0, 1, scm_array_map_x);
 
+SCM_SYMBOL (sym_b, "b");
 
 SCM_DEFINE (scm_array_map_x, "array-map!", 2, 0, 1,
 	    (SCM ra0, SCM proc, SCM lra),
@@ -811,12 +832,7 @@ SCM_DEFINE (scm_array_map_x, "array-map!", 2, 0, 1,
 {
   SCM_VALIDATE_PROC (2, proc);
   SCM_VALIDATE_REST_ARGUMENT (lra);
-  /* This is done as a test on lra, rather than an extra mandatory parameter
-     eval could check, so that the prototype for scm_array_map_x stays as it
-     was in the past.  scm_array_map_x isn't actually documented, but did
-     get a mention in the NEWS file, so is best left alone.  */
-  if (scm_is_null (lra))
-    SCM_WRONG_NUM_ARGS ();
+
   switch (SCM_TYP7 (proc))
     {
     default:
@@ -824,19 +840,29 @@ SCM_DEFINE (scm_array_map_x, "array-map!", 2, 0, 1,
  scm_ramapc (ramap, proc, ra0, lra, FUNC_NAME);
  return SCM_UNSPECIFIED;
     case scm_tc7_subr_1:
+      if (! scm_is_pair (lra))
+        SCM_WRONG_NUM_ARGS ();  /* need 1 source */
       scm_ramapc (ramap_1, proc, ra0, lra, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_subr_2:
+      if (! (scm_is_pair (lra) && scm_is_pair (SCM_CDR (lra))))
+        SCM_WRONG_NUM_ARGS ();  /* need 2 sources */
+      goto subr_2o;
     case scm_tc7_subr_2o:
+      if (! scm_is_pair (lra))
+        SCM_WRONG_NUM_ARGS ();  /* need 1 source */
+    subr_2o:
       scm_ramapc (ramap_2o, proc, ra0, lra, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_dsubr:
+      if (! scm_is_pair (lra))
+        SCM_WRONG_NUM_ARGS ();  /* need 1 source */
       scm_ramapc (ramap_dsubr, proc, ra0, lra, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_rpsubr:
       {
 	ra_iproc *p;
-	if (scm_is_false (scm_array_p (ra0, SCM_BOOL_T)))
+	if (!scm_is_typed_array (ra0, sym_b))
 	  goto gencase;
 	scm_array_fill_x (ra0, SCM_BOOL_T);
 	for (p = ra_rpsubrs; p->name; p++)
@@ -981,14 +1007,7 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
   unsigned long i;
   SCM_VALIDATE_PROC (2, proc);
 
-  if (scm_is_generalized_vector (ra))
-    {
-      size_t length = scm_c_generalized_vector_length (ra);
-      for (i = 0; i < length; i++)
-	GVSET (ra, i, scm_call_1 (proc, scm_from_ulong (i)));
-      return SCM_UNSPECIFIED;
-    }
-  else if (SCM_I_ARRAYP (ra))
+  if (SCM_I_ARRAYP (ra))
     {
       SCM args = SCM_EOL;
       int j, k, kmax = SCM_I_ARRAY_NDIM (ra) - 1;
@@ -997,10 +1016,10 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
       if (kmax < 0)
 	return scm_array_set_x (ra, scm_call_0 (proc), SCM_EOL);
 
-      scm_frame_begin (0);
+      scm_dynwind_begin (0);
 
       vinds = scm_malloc (sizeof(long) * SCM_I_ARRAY_NDIM (ra));
-      scm_frame_free (vinds);
+      scm_dynwind_free (vinds);
 
       for (k = 0; k <= kmax; k++)
 	vinds[k] = SCM_I_ARRAY_DIMS (ra)[k].lbnd;
@@ -1032,10 +1051,17 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
 	}
       while (k >= 0);
 
-      scm_frame_end ();
+      scm_dynwind_end ();
       return SCM_UNSPECIFIED;
     }
-  else
+  else if (scm_is_generalized_vector (ra))
+    {
+      size_t length = scm_c_generalized_vector_length (ra);
+      for (i = 0; i < length; i++)
+	GVSET (ra, i, scm_call_1 (proc, scm_from_ulong (i)));
+      return SCM_UNSPECIFIED;
+    }
+  else 
     scm_wrong_type_arg_msg (NULL, 0, ra, "array");
 }
 #undef FUNC_NAME
