@@ -1,23 +1,28 @@
 /* srfi-13.c --- SRFI-13 procedures for Guile
  *
- * Copyright (C) 2001, 2004, 2005, 2006 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2004, 2005, 2006, 2008 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <string.h>
 #include <ctype.h>
 
@@ -42,6 +47,18 @@
 			      start, &c_start, end, &c_end);        \
   } while (0)
 
+/* Expecting "unsigned char *c_str" */
+#define MY_VALIDATE_SUBSTRING_SPEC_UCOPY(pos_str, str, c_str,           \
+                                         pos_start, start, c_start,     \
+                                         pos_end, end, c_end)           \
+  do {                                                                  \
+    const char *signed_c_str;                                           \
+    MY_VALIDATE_SUBSTRING_SPEC_COPY(pos_str, str, signed_c_str,         \
+                                    pos_start, start, c_start,          \
+                                    pos_end, end, c_end);               \
+    c_str = (unsigned char *) signed_c_str;                             \
+  } while (0)
+
 #define MY_VALIDATE_SUBSTRING_SPEC(pos_str, str,              \
                                    pos_start, start, c_start, \
                                    pos_end, end, c_end)       \
@@ -994,16 +1011,16 @@ SCM_DEFINE (scm_string_compare, "string-compare", 5, 4, 0,
 	    "@var{i} is the first position that does not match.")
 #define FUNC_NAME s_scm_string_compare
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
   SCM proc;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   6, start1, cstart1,
-				   7, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   8, start2, cstart2,
-				   9, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    6, start1, cstart1,
+                                    7, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    8, start2, cstart2,
+                                    9, end2, cend2);
   SCM_VALIDATE_PROC (3, proc_lt);
   SCM_VALIDATE_PROC (4, proc_eq);
   SCM_VALIDATE_PROC (5, proc_gt);
@@ -1048,16 +1065,16 @@ SCM_DEFINE (scm_string_compare_ci, "string-compare-ci", 5, 4, 0,
 	    "character comparison is done case-insensitively.")
 #define FUNC_NAME s_scm_string_compare_ci
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
   SCM proc;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   6, start1, cstart1,
-				   7, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   8, start2, cstart2,
-				   9, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    6, start1, cstart1,
+                                    7, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    8, start2, cstart2,
+                                    9, end2, cend2);
   SCM_VALIDATE_PROC (3, proc_lt);
   SCM_VALIDATE_PROC (4, proc_eq);
   SCM_VALIDATE_PROC (5, proc_gt);
@@ -1181,15 +1198,15 @@ SCM_DEFINE (scm_string_lt, "string<", 2, 4, 0,
 	    "true value otherwise.")
 #define FUNC_NAME s_scm_string_lt
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
@@ -1224,15 +1241,15 @@ SCM_DEFINE (scm_string_gt, "string>", 2, 4, 0,
 	    "true value otherwise.")
 #define FUNC_NAME s_scm_string_gt
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
@@ -1267,15 +1284,15 @@ SCM_DEFINE (scm_string_le, "string<=", 2, 4, 0,
 	    "value otherwise.")
 #define FUNC_NAME s_scm_string_le
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
@@ -1310,15 +1327,15 @@ SCM_DEFINE (scm_string_ge, "string>=", 2, 4, 0,
 	    "otherwise.")
 #define FUNC_NAME s_scm_string_ge
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
@@ -1442,15 +1459,15 @@ SCM_DEFINE (scm_string_ci_lt, "string-ci<", 2, 4, 0,
 	    "case-insensitively.")
 #define FUNC_NAME s_scm_string_ci_lt
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
@@ -1486,15 +1503,15 @@ SCM_DEFINE (scm_string_ci_gt, "string-ci>", 2, 4, 0,
 	    "case-insensitively.")
 #define FUNC_NAME s_scm_string_ci_gt
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
@@ -1530,15 +1547,15 @@ SCM_DEFINE (scm_string_ci_le, "string-ci<=", 2, 4, 0,
 	    "case-insensitively.")
 #define FUNC_NAME s_scm_string_ci_le
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
@@ -1574,15 +1591,15 @@ SCM_DEFINE (scm_string_ci_ge, "string-ci>=", 2, 4, 0,
 	    "case-insensitively.")
 #define FUNC_NAME s_scm_string_ci_ge
 {
-  const char *cstr1, *cstr2;
+  const unsigned char *cstr1, *cstr2;
   size_t cstart1, cend1, cstart2, cend2;
 
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (1, s1, cstr1,
-				   3, start1, cstart1,
-				   4, end1, cend1);
-  MY_VALIDATE_SUBSTRING_SPEC_COPY (2, s2, cstr2,
-				   5, start2, cstart2,
-				   6, end2, cend2);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (1, s1, cstr1,
+                                    3, start1, cstart1,
+                                    4, end1, cend1);
+  MY_VALIDATE_SUBSTRING_SPEC_UCOPY (2, s2, cstr2,
+                                    5, start2, cstart2,
+                                    6, end2, cend2);
 
   while (cstart1 < cend1 && cstart2 < cend2)
     {
