@@ -1,22 +1,23 @@
 /* Copyright (C) 1996,1997,1998,1999,2000,2001, 2003, 2004, 2006, 2007 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
@@ -364,7 +365,7 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
 	{
 	  SCM vtable = SCM_STRUCT_VTABLE (chain);
 	  if (SCM_STRUCT_GC_CHAIN (vtable) != 0 && vtable != chain)
-	    SCM_SET_GC_MARK (vtable);
+	    SCM_SET_STRUCT_MARK (vtable);
 	  chain = SCM_STRUCT_GC_CHAIN (chain);
 	}
       /* Free unmarked structs.  */
@@ -374,9 +375,9 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
 	{
 	  SCM obj = chain;
 	  chain = SCM_STRUCT_GC_CHAIN (chain);
-	  if (SCM_GC_MARK_P (obj))
+	  if (SCM_STRUCT_MARK_P (obj))
 	    {
-	      SCM_CLEAR_GC_MARK (obj);
+	      SCM_CLEAR_STRUCT_MARK (obj);
 	      SCM_SET_STRUCT_GC_CHAIN (obj, newchain);
 	      newchain = obj;
 	    }
@@ -449,8 +450,14 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
       if (! SCM_LAYOUT_TAILP (SCM_CHAR (last_char)))
         goto bad_tail;
     }
-    
-  SCM_CRITICAL_SECTION_START;
+
+  /* In guile 1.8.5 and earlier, everything below was covered by a
+     CRITICAL_SECTION lock.  This can lead to deadlocks in garbage
+     collection, since other threads might be holding the heap_mutex, while
+     sleeping on the CRITICAL_SECTION lock.  There does not seem to be any
+     need for a lock on the section below, as it does not access or update
+     any globals, so the critical section has been removed. */
+
   if (SCM_STRUCT_DATA (vtable)[scm_struct_i_flags] & SCM_STRUCTF_ENTITY)
     {
       data = scm_alloc_struct (basic_size + tail_elts,
@@ -466,15 +473,7 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
   handle = scm_double_cell ((((scm_t_bits) SCM_STRUCT_DATA (vtable))
 			     + scm_tc3_struct),
 			    (scm_t_bits) data, 0, 0);
-  SCM_CRITICAL_SECTION_END;
 
-  /* In guile 1.8.1 and earlier, the SCM_CRITICAL_SECTION_END above covered
-     also the following scm_struct_init.  But that meant if scm_struct_init
-     finds an invalid type for a "u" field then there's an error throw in a
-     critical section, which results in an abort().  Not sure if we need any
-     protection across scm_struct_init.  The data array contains garbage at
-     this point, but until we return it's not visible to anyone except
-     `gc'.  */
   scm_struct_init (handle, layout, data, tail_elts, init);
 
   return handle;
@@ -897,8 +896,8 @@ scm_struct_prehistory ()
 {
   scm_i_structs_to_free = SCM_EOL;
   scm_c_hook_add (&scm_before_sweep_c_hook, scm_struct_gc_init, 0, 0);
-  /* With the new lazy sweep GC, the point at which the entire heap is
-     swept is just before the mark phase. */
+  /* With lazy sweep GC, the point at which the entire heap is swept
+     is just before the mark phase. */
   scm_c_hook_add (&scm_before_mark_c_hook, scm_free_structs, 0, 0);
 }
 
