@@ -3,21 +3,22 @@
 #ifndef SCM_THREADS_H
 #define SCM_THREADS_H
 
-/* Copyright (C) 1996,1997,1998,2000,2001, 2002, 2003, 2004, 2006 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,1998,2000,2001, 2002, 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 
@@ -50,9 +51,16 @@ typedef struct scm_i_thread {
 
   SCM handle;
   scm_i_pthread_t pthread;
-  
+
+  SCM cleanup_handler;
   SCM join_queue;
+
+  scm_i_pthread_mutex_t admin_mutex;
+  SCM mutexes;
+  scm_i_pthread_mutex_t *held_mutex;
+
   SCM result;
+  int canceled;
   int exited;
 
   SCM sleep_object;
@@ -61,10 +69,17 @@ typedef struct scm_i_thread {
   int sleep_fd, sleep_pipe[2];
 
   /* This mutex represents this threads right to access the heap.
-     That right can temporarily be taken away by the GC.  
+     That right can temporarily be taken away by the GC.
   */
   scm_i_pthread_mutex_t heap_mutex;
 
+  /* Boolean tracking whether the above mutex is currently locked by
+     this thread.  This is equivalent to whether or not the thread is
+     in "Guile mode".  This field doesn't need any protection because
+     it is only ever set or tested by the owning thread.
+  */
+  int heap_mutex_locked_by_self;
+
   /* The freelists of this thread.  Each thread has its own lists so
      that they can all allocate concurrently.
   */
@@ -83,7 +98,7 @@ typedef struct scm_i_thread {
    */
   SCM active_asyncs;            /* The thunks to be run at the next
                                    safe point */
-  unsigned int block_asyncs;    /* Non-zero means that asyncs should 
+  unsigned int block_asyncs;    /* Non-zero means that asyncs should
                                    not be run. */
   unsigned int pending_asyncs;  /* Non-zero means that asyncs might be pending.
 				 */
@@ -105,6 +120,7 @@ typedef struct scm_i_thread {
   SCM_STACKITEM *continuation_base;
 
   /* For keeping track of the stack and registers. */
+  SCM vm;
   SCM_STACKITEM *base;
   SCM_STACKITEM *top;
   jmp_buf regs;
@@ -131,23 +147,23 @@ SCM_API SCM scm_spawn_thread (scm_t_catch_body body, void *body_data,
 SCM_API void *scm_without_guile (void *(*func)(void *), void *data);
 SCM_API void *scm_with_guile (void *(*func)(void *), void *data);
 
-SCM_API void *scm_i_with_guile_and_parent (void *(*func)(void *), void *data,
-					   SCM parent);
+SCM_INTERNAL void *scm_i_with_guile_and_parent (void *(*func)(void *),
+						void *data, SCM parent);
 
 
 extern int scm_i_thread_go_to_sleep;
 
-void scm_i_thread_put_to_sleep (void);
-void scm_i_thread_wake_up (void);
-void scm_i_thread_invalidate_freelists (void);
+SCM_INTERNAL void scm_i_thread_put_to_sleep (void);
+SCM_INTERNAL void scm_i_thread_wake_up (void);
+SCM_INTERNAL void scm_i_thread_invalidate_freelists (void);
 void scm_i_thread_sleep_for_gc (void);
 
-void scm_threads_prehistory (SCM_STACKITEM *);
-void scm_threads_init_first_thread (void);
-SCM_API void scm_threads_mark_stacks (void);
-SCM_API void scm_init_threads (void);
-SCM_API void scm_init_thread_procs (void);
-SCM_API void scm_init_threads_default_dynamic_state (void);
+SCM_INTERNAL void scm_threads_prehistory (SCM_STACKITEM *);
+SCM_INTERNAL void scm_threads_init_first_thread (void);
+SCM_INTERNAL void scm_threads_mark_stacks (void);
+SCM_INTERNAL void scm_init_threads (void);
+SCM_INTERNAL void scm_init_thread_procs (void);
+SCM_INTERNAL void scm_init_threads_default_dynamic_state (void);
 
 
 #define SCM_THREAD_SWITCHING_CODE \
@@ -158,14 +174,26 @@ do { \
 
 SCM_API SCM scm_call_with_new_thread (SCM thunk, SCM handler);
 SCM_API SCM scm_yield (void);
+SCM_API SCM scm_cancel_thread (SCM t);
+SCM_API SCM scm_set_thread_cleanup_x (SCM thread, SCM proc);
+SCM_API SCM scm_thread_cleanup (SCM thread);
 SCM_API SCM scm_join_thread (SCM t);
+SCM_API SCM scm_join_thread_timed (SCM t, SCM timeout, SCM timeoutval);
+SCM_API SCM scm_thread_p (SCM t);
 
 SCM_API SCM scm_make_mutex (void);
 SCM_API SCM scm_make_recursive_mutex (void);
+SCM_API SCM scm_make_mutex_with_flags (SCM flags);
 SCM_API SCM scm_lock_mutex (SCM m);
+SCM_API SCM scm_lock_mutex_timed (SCM m, SCM timeout, SCM owner);
 SCM_API void scm_dynwind_lock_mutex (SCM mutex);
 SCM_API SCM scm_try_mutex (SCM m);
 SCM_API SCM scm_unlock_mutex (SCM m);
+SCM_API SCM scm_unlock_mutex_timed (SCM m, SCM cond, SCM timeout);
+SCM_API SCM scm_mutex_p (SCM o);
+SCM_API SCM scm_mutex_locked_p (SCM m);
+SCM_API SCM scm_mutex_owner (SCM m);
+SCM_API SCM scm_mutex_level (SCM m);
 
 SCM_API SCM scm_make_condition_variable (void);
 SCM_API SCM scm_wait_condition_variable (SCM cond, SCM mutex);
@@ -173,6 +201,7 @@ SCM_API SCM scm_timed_wait_condition_variable (SCM cond, SCM mutex,
 					       SCM abstime);
 SCM_API SCM scm_signal_condition_variable (SCM cond);
 SCM_API SCM scm_broadcast_condition_variable (SCM cond);
+SCM_API SCM scm_condition_variable_p (SCM o);
 
 SCM_API SCM scm_current_thread (void);
 SCM_API SCM scm_all_threads (void);
@@ -192,7 +221,7 @@ SCM_API scm_i_pthread_key_t scm_i_thread_key;
 #define scm_i_set_last_debug_frame(f) \
                                  (SCM_I_CURRENT_THREAD->last_debug_frame = (f))
 
-SCM_API scm_i_pthread_mutex_t scm_i_misc_mutex;
+SCM_INTERNAL scm_i_pthread_mutex_t scm_i_misc_mutex;
 
 /* Convenience functions for working with the pthread API in guile
    mode.
@@ -205,7 +234,7 @@ SCM_API int scm_pthread_cond_wait (pthread_cond_t *cond,
 				   pthread_mutex_t *mutex);
 SCM_API int scm_pthread_cond_timedwait (pthread_cond_t *cond,
 					pthread_mutex_t *mutex,
-					const struct timespec *abstime);
+					const scm_t_timespec *abstime);
 #endif
 
 /* More convenience functions.
